# Load packages -----------------------------------------------------
library(shiny)
library(ggplot2)
library(dplyr)
library(tools)
library(stringr)

# Load data ---------------------------------------------------------
load("data/movies.Rdata")

# Define UI ---------------------------------------------------------
ui <- fluidPage(
  
  # App title
  titlePanel("Movie browser"),
  
  # Sidebar layout with a input and output definitions
  sidebarLayout(
    
    # Inputs: Select variables to plot
    sidebarPanel(
      
      # Select variable for y-axis
      selectInput(inputId = "y", 
                  label = "Y-axis:",
                  choices = c("IMDB rating" = "imdb_rating", 
                              "IMDB number of votes" = "imdb_num_votes", 
                              "Critics Score" = "critics_score", 
                              "Audience Score" = "audience_score", 
                              "Runtime" = "runtime"), 
                  selected = "audience_score"),
      
      # Select variable for x-axis
      selectInput(inputId = "x", 
                  label = "X-axis:",
                  choices = c("IMDB rating" = "imdb_rating", 
                              "IMDB number of votes" = "imdb_num_votes", 
                              "Critics Score" = "critics_score", 
                              "Audience Score" = "audience_score", 
                              "Runtime" = "runtime"), 
                  selected = "critics_score"),
      
      # Select variable for color
      selectInput(inputId = "z", 
                  label = "Color by:",
                  choices = c("Title Type" = "title_type", 
                              "Genre" = "genre", 
                              "MPAA Rating" = "mpaa_rating", 
                              "Critics Rating" = "critics_rating", 
                              "Audience Rating" = "audience_rating"),
                  selected = "mpaa_rating"),
      
      # Set alpha level
      sliderInput(inputId = "alpha", 
                  label = "Alpha:", 
                  min = 0, max = 1, 
                  value = 0.5),
      
      # Show data table
      checkboxInput(inputId = "show_data",
                    label = "Show data table",
                    value = TRUE),
      
      # Enter text for plot title, only when other inputs are updated
      textInput(inputId = "plot_title", 
                label = "Plot title", 
                placeholder = "Enter text to be used as plot title"),
      
      # Enter text for description, only when action button is clicked
      textInput(inputId = "plot_description", 
                label = "Plot description", 
                placeholder = "Describe what you see in the plot"),
      
      # Action button to update text for description
      actionButton(inputId = "update_plot_description", label = "Update plot description"),
      
      # Horizontal line for visual separation
      hr(),
      
      # Select which types of movies to plot
      checkboxGroupInput(inputId = "selected_type",
                         label = "Select movie type(s):",
                         choices = c("Documentary", "Feature Film", "TV Movie"),
                         selected = "Feature Film")
    ),
    
    # Output:
    mainPanel(
      
      # Show scatterplot
      plotOutput(outputId = "scatterplot"),
      br(),    # a little bit of visual separation
      
      # Print number of obs plotted
      uiOutput(outputId = "n"),
      br(),    # a little bit of visual separation
      
      # Print plot description
      uiOutput(outputId = "plot_description_text"),
      br(),    # a little bit of visual separation

      # Show data table
      DT::dataTableOutput(outputId = "moviestable")
    )
  )
)

# Define server function --------------------------------------------
server <- function(input, output) {
  
  # Create a subset of data filtering for selected title types
  movies_subset <- reactive({
    req(input$selected_type) # ensure availability of value before proceeding
    filter(movies, title_type %in% input$selected_type)
  })
  
  # Create scatterplot object the plotOutput function is expecting
  output$scatterplot <- renderPlot({
    ggplot(data = movies_subset(), aes_string(x = input$x, y = input$y,
                                              color = input$z)) +
      geom_point(alpha = input$alpha) +
      labs(x = toTitleCase(str_replace_all(input$x, "_", " ")),
           y = toTitleCase(str_replace_all(input$y, "_", " ")),
           color = toTitleCase(str_replace_all(input$z, "_", " ")),
           # With isolate() only update plot title when other inputs that
           # go into the plot change, not when the title is updated
           title = isolate({ toTitleCase(input$plot_title) })
           )
  })
  
  # Print number of movies plotted
  output$n <- renderUI({
    types <- movies_subset()$title_type %>% 
      factor(levels = input$selected_type) 
    counts <- table(types)
    
    HTML(paste("There are", counts, input$selected_type, "movies in this dataset. <br>"))
  })
  
  # Print plot description
  # With eventReactive() only update plot description when 
  # action button is clicked
  output$plot_description_text <- eventReactive(
    eventExpr = input$update_plot_description,
    valueExpr = { HTML(input$plot_description) }
    )
  
  # Print data table if checked
  output$moviestable <- DT::renderDataTable(
    if(input$show_data){
      DT::datatable(data = movies_subset()[, 1:7], 
                    options = list(pageLength = 10), 
                    rownames = FALSE)
    }
  )
}

# Create the Shiny app object ---------------------------------------
shinyApp(ui, server)
